<?php

namespace App\Console\Commands;

use Carbon\Carbon;
use Illuminate\Console\Command;
use Illuminate\Support\Facades\Storage;

use App\Classes\File;
use App\Classes\FTN\Packet;
use App\Jobs\MessageProcess as Job;
use App\Models\Address;

class PacketProcess extends Command
{
	/**
	 * The name and signature of the console command.
	 *
	 * @var string
	 */
	protected $signature = 'packet:process'
		.' {file : Packet to process}'
		.' {--N|nobot : Dont process bots}'
		.' {ftn? : System the packet is from}';

	/**
	 * The console command description.
	 *
	 * @var string
	 */
	protected $description = 'Process Packet';

	/**
	 * Execute the console command.
	 *
	 * @return mixed
	 * @throws \App\Classes\FTN\InvalidPacketException
	 * @todo Should this just call PacketProcess instead?
	 */
	public function handle()
	{
		$fs = Storage::disk(config('fido.local_disk'));
		$rel_name = sprintf('%s/%s',config('fido.dir'),$this->argument('file'));
		$a = NULL;

		$f = new File($fs->path($rel_name));

		$m = NULL;
		if ($this->argument('ftn')) {
			$a = Address::findFTN($this->argument('ftn'));

		} elseif (preg_match('/^(([0-9A-F]+)-([0-9]+))+/',$this->argument('file'),$m)) {
			$a = Address::findOrFail(hexdec($m[2]));

		} else {
			$this->error('Unable to determine sender FTN address');
			exit(1);
		}

		foreach ($f as $packet) {
			foreach ($pkt = Packet::process($packet,$f->itemName(),$f->itemSize(),$a?->zone->domain) as $msg) {
				// @todo Quick check that the packet should be processed by us.
				$this->info(sprintf('Processing message from [%s] with msgid [%s] in (%s)',$msg->fboss,$msg->msgid,$f->pktName()));

				// Dispatch job.
				Job::dispatch($msg,$f->pktName(),$a,$pkt->fftn_o,Carbon::now(),$this->option('nobot'));
			}
		}
	}
}